﻿//	****************************************************
//	EasyTween
//	****************************************************
//	By: Daniel Holth
//
//	EasyTween extends mx.transitions.Tween 
//	
//	EXAMPLE USAGE:
//	import com.fedins.utils.EasyTween;
//  import flash.filters.*;
//
//	The following code will move a MovieClip with the instance name 'aMovieClip_mc' to 
//	point 50,50 on the stage and rotate it to 90 degrees in 12 frames. 
//
//	var easyTween:EasyTween = new EasyTween(aMovieClip_mc, "moveTween", [50, 50, 90], 12,  mx.transitions.easing.Bounce.easeOut);
//
//  Open source ExtendedTween code By George Medve <george [at] sqcircle.com>
//
//	****************************************************
//	Last Modified 10/10/2006
//	****************************************************
//
/////////////////////////////////////////////
//	IMPORTS
/////////////////////////////////////////////
/*
import flash.filters.BitmapFilter;
import flash.filters.BlurFilter;
import flash.filters.GlowFilter;
import flash.filters.DropShadowFilter;
import flash.filters.BevelFilter;
import flash.filters.ColorMatrixFilter;
*/
import mx.transitions.Tween;
import mx.transitions.BroadcasterMX;
import mx.transitions.OnEnterFrameBeacon;
/////////////////////////////////////////////
/////////////////////////////////////////////

class util.EasyTween extends Tween{
	public var className:String = "EasyTween";
	public static var version:String = "1.0.0";
	
	//
	//	LISTENERS
	public var onMotionStarted:Function;
	public var onMotionStopped:Function;
	public var onMotionFinished:Function;
	public var onMotionChanged:Function;
	public var onMotionResumed:Function;
	public var onMotionLooped:Function;
	//
	public var aProps:Array;
	public var aBegin:Array;
	public var aChange:Array;
	public var aPrevPos:Array;
	//
	
	private var point1:Number;
	private var point2:Number;
	//
	// _aPos is my equivalent of _pos, in both cases it seems to have no purpose
	private var _aPos:Array;
	// 
	
	/////////////////////////////////////////////
	//	CONSTRUCTOR
	///////////////////////////////////////////// 
		// mc:  MovieClip to be tweened
		// tweenType:	what sort of tween it.  Chose between moveTween, scaleTween, alphaTween, filterTween and tintColorTween
		// args: Array of parameters for the tween. See specific methods for more detail
		// dur: Duration of frames for the tween.  Defaults to 8
		// easeFunc: easing type.  Defaults to mx.transitions.easing.Regular.easeOut
		// useSeconds: duration will default to frames.  Setting this to true means dur = nbr of seconds.
	public function EasyTween(mc:MovieClip, tweenType:String, args:Array, dur:Number, easeFunc:Function, useSeconds:Boolean){
		if(mc == undefined){
			throw new Error("ERROR: " + this  + " | Undefined target MovieClip");
		}
		if(easeFunc == null or easeFunc == undefined){
			easeFunc = mx.transitions.easing.Regular.easeOut;
		}
		if(dur == null or dur == undefined){
			dur = 8;
		}
		if(useSeconds == null or undefined){
			useSeconds = false;
		}
		switch(tweenType){
			case "moveTween" :
				moveTween(mc, easeFunc, args, dur, useSeconds);
				break;
			case "scaleTween" :
				scaleTween(mc, easeFunc, args, dur, useSeconds);
				break;
			case "dimensionTween" :
				dimensionTween(mc, easeFunc, args, dur, useSeconds);
				break;
			case "alphaTween" :
				alphaTween(mc, easeFunc, args, dur, useSeconds);
				break;
				/*
			case "filterTween" :
				filterTween(mc, easeFunc, BitmapFilter(args[0]), dur, useSeconds);
				break;
			case "tintColorTween" :
				tintColorTween(mc, easeFunc, args, dur, useSeconds);
				break;
			case "brightnessTween" :
				brightnessTween(mc, easeFunc, Number(args[0]), dur, useSeconds);
				break;
				*/
			default :
				throw new Error("ERROR: " + this  + " | Unsupported tween type [" + tweenType + "]");
				break;
		}
	}
	
	public function kill() {
		
		var tw = this;
		tw.onMotionFinished = die;
	}
	
	public function die() {
		var tw = this;
		tw.onMotionFinished = null;
		tw.onEnterFrame = null;
		
	}

	
	/////////////////////////////////////////////
	/////////////////////////////////////////////
	// Moves the mc from its current location to another
	// args: [new_x_position, new_y_position, new_rotation] 
	private function moveTween(mc:MovieClip, easeFunc:Function, args:Array, dur:Number, useSeconds:Boolean):Void{
		tweenExtended(mc, ["_x", "_y", "_rotation"], easeFunc, [mc._x, mc._y, mc._rotation], args, dur, useSeconds);
	}
	
	// Scales the mc based on a percentage of normal size.
	// args: [new_x_scale, new_y_scale]
	private function scaleTween(mc:MovieClip, easeFunc:Function, args:Array, dur:Number, useSeconds:Boolean):Void{
		tweenExtended(mc, ["_xscale", "_yscale"], easeFunc, [mc._xscale, mc._yscale], args, dur, useSeconds);
	}
	
	// Changes the width and height of a clip
	// args: [new_width, new_height]
	private function dimensionTween(mc:MovieClip, easeFunc:Function, args:Array, dur:Number, useSeconds:Boolean):Void{
		tweenExtended(mc, ["_width", "_height"], easeFunc, [mc._width, mc._height], args, dur, useSeconds);
	}
	
	// Changes the _alpha value of the MC to the new value.
	// args: [new_alpha]
	private function alphaTween(mc:MovieClip, easeFunc:Function, args:Array, dur:Number, useSeconds:Boolean):Void{
		tweenExtended(mc, ["_alpha"], easeFunc, [mc._alpha], args, dur, useSeconds);
	}
	/*
	// Determines what filter to tween and runs the appropriate function.
	private function filterTween(mc:MovieClip, easeFunc:Function, f:BitmapFilter, dur:Number, useSeconds:Boolean):Void{
		switch(true){
			case f instanceof BlurFilter :
				blur(mc, easeFunc, BlurFilter(f), dur)
				break;
			case f instanceof GlowFilter :
				glow(mc, easeFunc, GlowFilter(f), dur);
				break;
			case f instanceof DropShadowFilter:
				dropShadow(mc, easeFunc, DropShadowFilter(f), dur);
				break;
			case f instanceof BevelFilter:
				bevel(mc, easeFunc, BevelFilter(f), dur);
				break;
			case f instanceof ColorMatrixFilter:
				colorMatrix(mc, easeFunc, ColorMatrixFilter(f), dur);
				break;
			default :
				throw new Error("ERROR: " + this  + " | Unsupported filter type [" + f + "]");
				break;
		
		}
	}

	// Determines if the passed filter is already on the mc so that the 
	// filterTween can adjust it from its current value.
	private function findFilter(mc:MovieClip, obj:BitmapFilter):Number{
		var l:Number = mc.filters.length;
		for(var i = 0; i < l; i++){
			if(mc.filters[i] instanceof obj){
				return i;
			}
		}
		return -1;
	}
	
	// Adds the filter to the movieclip's filter array the proper way.
	private function addFilter(mc:MovieClip, bf:BitmapFilter, i:Number):Void{
		var clone:Array = mc.filters;
		clone[i] = bf;
		mc.filters = clone;
	}
	
	// Blurs the MovieClip.
	// BlurFilter has the following parameters:
		// blurX: Number of pixels blurred in x direction.  Defaults to 4.
		// blurY: Number of pixels blurred in y direction.  Defaults to 4.
		// quality: Number of times to run the blur.  Range 0-15.  Default is 1.
	private function blur(mc:MovieClip, easeFunc:Function, f:BlurFilter, dur:Number, useSeconds:Boolean){
		var blurFilter:BlurFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, BlurFilter);
		if(filterFound == -1){
			filterFound = l;
			blurFilter = new BlurFilter(0, 0);
			if(l == 0){
				mc.filters = [blurFilter];
			} else {
				mc.filters.push(blurFilter);
			}
		} else {
			blurFilter = mc.filters[filterFound];
		}
		
		tweenExtended (blurFilter, ["blurX", "blurY"],  easeFunc, [blurFilter.blurX, blurFilter.blurY], [f.blurX, f.blurY], dur, useSeconds);
		var scope= this;
		onMotionChanged = function() {
			scope.addFilter(mc, blurFilter, filterFound);
    	};
	}
	
	// Adds a glow to the MovieClip.  Should be applied BEFORE any DropShadowFilter.
	// GlowFilter has the following parameters:
		// color: Hex value of the color of the glow.  Default is 0xFF0000 (red).
		// alpha: Alpha value of the glow.  Range 0-1.  Default is 1.
		// blurX: Number of pixels blurred in x direction.  Range 0-255. Defaults to 6.
		// blurY: Number of pixels blurred in y direction.  Range 0-255. Defaults to 6.
		// strength: The punch strength of the shadow. Range 0-255.  Defaults to 2.
		// quality: Number of times to run the blur.  Range 0-15.  Default is 1.
		// inner: true/false if it is in an inner.  Default is false.
		// knockout: true/false should the original mc be displayed.  Default is true.
	private function glow(mc:MovieClip, easeFunc:Function, f:GlowFilter, dur:Number, useSeconds:Boolean):Void{
		var filter:GlowFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, GlowFilter);
		if(filterFound == -1){
			filterFound = l;
			filter = new GlowFilter(f.color, 0);
			if(l == 0){
				mc.filters = [filter];
			} else {
				mc.filters.push(filter);
			}
		} else {
			filter = mc.filters[filterFound];
		}
		
		tweenExtended(filter, ["color", "alpha", "blurX", "blurY", "strength"], easeFunc, [filter.color, filter.alpha, filter.blurX, filter.blurY, filter.strength], [f.color, f.alpha, f.blurX, f.blurY, f.strength], dur, useSeconds);
		var scope= this;
		onMotionChanged = function() {
			scope.addFilter(mc, filter, filterFound);
    	};
	}
	
	// Adds a dropshadow to the MovieClip.
	// DropShadowFilter has the following parameters:
		// distance: distance in pixels shadow is offset from the object. Default is 4.
		// angle: angle of the light source.  Default is 45. (top left).
		// color: Hex value of the color of the shadow.  Default is 0x000000 (black).
		// alpha: Alpha value of the shadow.  Range 0-1.  Default is 1.
		// blurX: Number of pixels blurred in x direction.  Range 0-255. Defaults to 4.
		// blurY: Number of pixels blurred in y direction.  Range 0-255. Defaults to 4.
		// strength: The punch strength of the shadow. Range 0-255.  Defaults to 1.
		// quality: Number of times to run the blur on the shadow.  Range 0-15.  Default is 1.
		// inner: true/false if it is in an inner shadow.  Default is false.
		// knockout: true/false should the original mc be displayed.  Default is true.
		// hideObject: true/false hides the original object.  Defalut is false.
	private function dropShadow(mc:MovieClip, easeFunc:Function, f:DropShadowFilter, dur:Number, useSeconds:Boolean):Void{
		var filter:DropShadowFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, DropShadowFilter);
		if(filterFound == -1){
			filterFound = l;
			filter = new DropShadowFilter(0, 0, f.color);
			if(l == 0){
				mc.filters = [filter];
			} else {
				mc.filters.push(filter);
			}
		} else {
			filter = mc.filters[filterFound];
		}
		
		tweenExtended (filter, ["distance", "angle", "color", "alpha", "blurX", "blurY", "strength"], easeFunc, [filter.distance, filter.angle, filter.color, filter.alpha, filter.blurX, filter.blurY, filter.strength], [f.distance, f.angle, f.color, f.alpha, f.blurX, f.blurY, f.strength], dur, useSeconds);
		var scope= this;
		onMotionChanged = function() {
			scope.addFilter(mc, filter, filterFound);
    	};
	}
	
	// Adds a bevel to the movieclip.
	// BevelFilter has the following parameters:
		// distance: distance in pixels the bevel is offset. Default is 4.
		// angle: angle of the light source.  Default is 45. (top left).
		// highlightColor: Hex value of the color of the highlights.  Default is 0xFFFFFF (white).
		// highlightAlpha: Alpha value of the highlights.  Range 0-1.  Default is 1.
		// shadowColor: Hex value of the color of the shadows.  Default is 0x000000 (black).
		// shadowAlpha: Alpha value of the shadow.  Range 0-1.  Default is 1.
		// blurX: Number of pixels blurred in x direction.  Range 0-255. Defaults to 4.
		// blurY: Number of pixels blurred in y direction.  Range 0-255. Defaults to 4.
		// strength: The punch strength of the bevel.  Defaults to 1.
		// quality: Number of times to run the blur on the bevel.  Range 0-15.  Default is 1.
		// type: Type of the bevel.  Constants: inner, outer, full.  Default is inner.
		// knockout: true/false should the original mc be displayed.  Default is true.
	private function bevel(mc:MovieClip, easeFunc:Function, f:BevelFilter, dur:Number, useSeconds:Boolean):Void{
		var filter:BevelFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, BevelFilter);
		if(filterFound == -1){
			filterFound = l;
			filter = new BevelFilter(0, 0, f.highlightColor, f.highlightAlpha, f.shadowColor, f.shadowAlpha);
			if(l == 0){
				mc.filters = [filter];
			} else {
				mc.filters.push(filter);
			}
		} else {
			filter = mc.filters[filterFound];
		}
		
		tweenExtended (filter, ["distance", "angle", "highlightColor", "highlightAlpha", "shadowColor", "shadowAlpha", "blurX", "blurY", "strength"], easeFunc, [filter.distance, filter.angle, filter.highlightColor, filter.highlightAlpha, filter.shadowColor, filter.shadowAlpha, filter.blurX, filter.blurY, filter.strength], [f.distance, f.angle, f.highlightColor, f.highlightAlpha, f.shadowColor, f.shadowAlpha, f.blurX, f.blurY, f.strength], dur, useSeconds);
		var scope= this;
		onMotionChanged = function() {
			scope.addFilter(mc, filter, filterFound);
    	};
	}

	// makes an associative array object to create an array for the CMF.
	private function makeColorArrayObject(filter:ColorMatrixFilter):Object{
		var c:Object = new Object();
		c.a = filter.matrix[0];
		c.b = filter.matrix[1];
		c.c = filter.matrix[2];
		c.d = filter.matrix[3];
		c.e = filter.matrix[4];
		c.f = filter.matrix[5];
		c.g = filter.matrix[6];
		c.h = filter.matrix[7];
		c.i = filter.matrix[8];
		c.j = filter.matrix[9];
		c.k = filter.matrix[10];
		c.l = filter.matrix[11];
		c.m = filter.matrix[12];
		c.n = filter.matrix[13];
		c.o = filter.matrix[14];
		c.p = filter.matrix[15];
		c.q = filter.matrix[16];
		c.r = filter.matrix[17];
		c.s = filter.matrix[18];
		c.t = filter.matrix[19];
		return c;
	}
	
	// makes a new CMF and populates its matrix
	private function makeColorMatrix(c:Object):ColorMatrixFilter{
		var filter:ColorMatrixFilter = new ColorMatrixFilter();
		filter.matrix[0] = c.a;
		filter.matrix[1] = c.b; 
		filter.matrix[2] = c.c; 
		filter.matrix[3] = c.d; 
		filter.matrix[4] = c.e; 
		filter.matrix[5] = c.f; 
		filter.matrix[6] = c.g; 
		filter.matrix[7] = c.h; 
		filter.matrix[8] = c.i; 
		filter.matrix[9] = c.j; 
		filter.matrix[10] = c.k; 
		filter.matrix[11] = c.l; 
		filter.matrix[12] = c.m; 
		filter.matrix[13] = c.n; 
		filter.matrix[14] = c.o; 
		filter.matrix[15] = c.p; 
		filter.matrix[16] = c.q; 
		filter.matrix[17] = c.r; 
		filter.matrix[18] = c.s; 
		filter.matrix[19] = c.t; 
		return filter;
	}

	// changes the color of the MovieClip.
	// ColorMatrixFilter has the following parameters:
		// matrix: an array of 20 values to adjust the color.
	private function colorMatrix(mc:MovieClip, easeFunc:Function, f:ColorMatrixFilter, dur:Number, useSeconds:Boolean):Void{
		var filter:ColorMatrixFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, ColorMatrixFilter);
		if(filterFound == -1){
			filterFound = l;
			filter = new ColorMatrixFilter();
			if(l == 0){
				mc.filters = [filter];
			} else {
				mc.filters.push(filter);
			}
		} else {
			filter = mc.filters[filterFound];
		}	
		var c1:Object = makeColorArrayObject(f);
		var c2:Object = makeColorArrayObject(filter);
		
		tweenExtended (c1, ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t"],  easeFunc, [c2.a, c2.b, c2.c, c2.d, c2.e, c2.f, c2.g ,c2.h, c2.i, c2.j, c2.k, c2.l, c2.m, c2.n, c2.o, c2.p, c2.q, c2.r, c2.s, c2.t], [c1.a, c1.b, c1.c, c1.d, c1.e, c1.f, c1.g ,c1.h, c1.i, c1.j, c1.k, c1.l, c1.m, c1.n, c1.o, c1.p, c1.q, c1.r, c1.s, c1.t], dur, useSeconds);
		var scope= this;
		onMotionChanged = function() {
			var cmf:ColorMatrixFilter = new ColorMatrixFilter([c1.a, c1.b, c1.c, c1.d, c1.e,
															   c1.f, c1.g ,c1.h, c1.i, c1.j, 
															   c1.k, c1.l, c1.m, c1.n, c1.o, 
														       c1.p, c1.q, c1.r, c1.s, c1.t]);
			scope.addFilter(mc, cmf, filterFound);
    	};
	}
	
	// Methods to easily brighten or darken the MovieClip 
	// takes in the new brigthness.  range -100 to 100;
	private function brightnessTween(mc:MovieClip, easeFunc:Function, args:Number, dur:Number, useSeconds:Boolean):Void{
		var filter:ColorMatrixFilter;
		var l:Number = mc.filters.length;
		var filterFound:Number = findFilter(mc, ColorMatrixFilter);
		if(filterFound == -1){
			filterFound = l;
			filter = new ColorMatrixFilter();
			if(l == 0){
				mc.filters = [filter];
			} else {
				mc.filters.push(filter);
			}
		} else {
			filter = mc.filters[filterFound];
		}
		
		var cmf:ColorMatrixFilter = new ColorMatrixFilter([filter.matrix[0], filter.matrix[1], filter.matrix[2], filter.matrix[3], args,
													  	   filter.matrix[5], filter.matrix[6], filter.matrix[7], filter.matrix[8], args, 
													   	   filter.matrix[10], filter.matrix[11], filter.matrix[12], filter.matrix[13], args, 
													       filter.matrix[15], filter.matrix[16], filter.matrix[17], filter.matrix[18], filter.matrix[19]]);
		
		this.filterTween(mc, easeFunc, cmf, dur, useSeconds)
	}

	
	// Methods to easily tint the MovieClip to the selected color.  
	// Second parameter is amount of tint on original color. 0-1, 0 meaning full color change, 1 no change.  Default is 0.
	private function tintColorTween(mc:MovieClip, easeFunc:Function, args:Array, dur:Number, useSeconds:Boolean):Void{
		var colorToChange:String = args[0];
		if(args[1] == undefined or args[1] == null){
			tint = 0;
		} else {
			var tint:Number = 1 - args[1];
		}
		switch(colorToChange){
			case "RED":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([1, 0, 0, 0, 0,
													  			   0, tint, 0, 0, 0, 
													   			   0, 0, tint, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "GREEN":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([tint, 0, 0, 0, 0,
													  			   0, 1, 0, 0, 0, 
													   			   0, 0, tint, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "BLUE":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([tint, 0, 0, 0, 0,
													  			   0, tint, 0, 0, 0, 
													   			   0, 0, 1, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "PURPLE":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([1, 0, 0, 0, 0,
													  			   0, tint, 0, 0, 0, 
													   			   0, 0, 1, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "YELLOW":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([1, 0, 0, 0, 0,
													  			   0, 1, 0, 0, 0, 
													   			   0, 0, tint, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "CYAN":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([tint, 0, 0, 0, 0,
													  			   0, 1, 0, 0, 0, 
													   			   0, 0, 1, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "GREY":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([.3086, .6094, .082, 0, 0,
													  			   .3086, .6094, .082, 0, 0, 
													   			   .3086, .6094, .082, 0, 0, 
													      	       .3086, 0, 0, 1, 0]);
				break;
			case "NEGATIVE":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([-1, 0, 0, 0, 255,
													  			   0, -1, 0, 0, 255, 
													   			   0, 0, -1, 0, 255, 
													      	       0, 0, 0, 1, 0]);
				break;
			case "ORIGINAL":
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([1, 0, 0, 0, 0,
													  			   0, 1, 0, 0, 0, 
													   			   0, 0, 1, 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			default :
				var c = hexToRgb(args[0]);
				var cmf:ColorMatrixFilter = new ColorMatrixFilter([(c.rb/255), 0, 0, 0, 0,
													  			   0, (c.gb/255), 0, 0, 0, 
													   			   0, 0, (c.bb/255), 0, 0, 
													      	       0, 0, 0, 1, 0]);
				break;
			
		}
		
		this.filterTween(mc, easeFunc, cmf, dur, useSeconds)
	}
		*/
		
	// Converts a hex number to its RGB equivilent.
	private function hexToRgb (hex:Number):Object{
		var r = hex >> 16;
		var temp = hex ^ r << 16;
		var g = temp >> 8;
		var b = temp ^ g << 8;
		return ({rb:r, gb:g, bb:b});
	}
	
	
	// an extended version of the tween method to handle multiple variables at once.
	private function tweenExtended (target:Object, props:Array, easeFunc:Function, strt:Array, end:Array, dur:Number, useSecs:Boolean, p1:Number, p2:Number):Void{
		OnEnterFrameBeacon.init ();
		if (!arguments.length) {
			return;
		}
		// 
		this.aChange = [];
		this.obj = target;
		this.aProps = props;
		this.aBegin = strt;
		this.position = copyArray (strt);
		this.duration = dur;
		this.useSeconds = useSecs;
		if(easeFunc) {
			this.func = easeFunc;
		}
		this.finish = end;
		
		this.point1 = p1;
		this.point2 = p2;
		this._listeners = [];
		// aPrevPos = [];
		// _aPos = [];
		addListener (this);
		this.start ();
		//	
	}
	
	private function deleteTween():Void {
		delete this;
	}
	
	/////////////////////////////////////////////
	//	OVERRIDER METHODS
	/////////////////////////////////////////////
	private function update ():Void {
		this.position = this.getPosition (this._time);
	}
	

	public function continueTo (fin:Array, dur:Number):Void {
		for (var i = 0; i < aProps.length; i++) {
			this.aBegin[i] = this.position[i];
		};
		
		finish = fin;
		if (dur != undefined) {
			this.duration = dur;
		};
		start();
	}
	
	public function yoyo ():Void {
		var aTemp:Array = this.copyArray (this.aBegin);
		this.continueTo (aTemp, this.time);
	}
	
	/////////////////////////////////////////////
	/////////////////////////////////////////////

	
	/////////////////////////////////////////////
	// 	GETTERS AND SETTERS
	/////////////////////////////////////////////
	public function get position ():Array {
		return this.getPosition ();
	}
	
	public function getPosition (t:Number):Array {
		if (t == undefined){
			t = this._time;
		}
		
		var aPos:Array = [];
		
		for (var i = 0; i < aProps.length; i++){
			var b:Number = Number (this.aBegin[i]);
			var c:Number = Number (this.aChange[i]);
			
			// Check for Bezier Points
			if ( (this.point1 == undefined) && (this.point2 == undefined)) {
				// No Bezier Points, normal calculation
				aPos[i] = this.func (t, b, c, this._duration);
			} else if ( (this.point1 != undefined) && (this.point2 == undefined)) {
				// Single Bezier point - tweenQuadBez or tweenQuadBezThru
				aPos[i] = this.func (t, b, c, this._duration, this.point1);
			} else if ( (this.point1 != undefined) && (this.point2 != undefined)) {
				// Two Bezier points - tweenCubicBez
				aPos[i] = this.func (t, b, c, this._duration, this.point1, this.point2);
			}
		}
		return aPos;
	}
	
	public function set position (p:Array):Void {
		this.setPosition (p);
	}
	
	public function setPosition (p:Array):Void {
		for (var i = 0; i < this.aProps.length; i++) {
			// aPrevPos[i] = _aPos[i]; seems to serve no purpose
			this.obj[aProps[i]] = this._aPos[i] = p[i];
		}
		this.broadcastMessage ("onMotionChanged", this, p);
		// added updateAfterEvent for setInterval-driven motion
		updateAfterEvent ();
	}
	
	public function set finish (f:Array):Void {
		for (var i = 0; i < aProps.length; i++) {
			this.aChange[i] = f[i] - this.aBegin[i];
		}
	}
	
	public function get finish ():Array {
		var aTemp = [];
		for (var i = 0; i < this.aProps.length; i++) {
			aTemp[i] = this.aBegin[i] + this.aChange[i];
		}
		return aTemp;
	}
	
	public function set loop (b:Boolean):Void {
		this.looping = b;
	}
	
	public function get loop ():Boolean {
		return this.looping;
	}
	/////////////////////////////////////////////
	/////////////////////////////////////////////
	
	
	/////////////////////////////////////////////
	//	TOOLS
	/////////////////////////////////////////////
	private function copyArray (arr:Array):Array {
		var aTemp:Array = [];
		if (arr.length == 0) {
			for (var i in arr) {
				aTemp[i] = arr[i];
			}
		}
		for (var i = 0; i < arr.length; i++) {
			aTemp[i] = arr[i];
		}
		return aTemp;
	}
	/////////////////////////////////////////////
	/////////////////////////////////////////////
	
	public function toString ():String {
		return ("[" + className + "]");
	}	
}